/*
 * SimpleApp Demo
 * 
 * Copyright (c) 2010, CodeRight
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice and this list of conditions.
 *   - Redistributions in binary form must reproduce the above
 *     copyright notice and this list of conditions in the 
 *     documentation and/or other materials provided with the 
 *     distribution.
 * 
 * Jazz Desktop Application Framework
 * http://jazz.coderight.nl
 * 
 * Author: Will Boluyt
 */
package nl.coderight.jazz.demo;

import java.awt.FlowLayout;
import java.util.ArrayList;

import javax.swing.SortOrder;

import nl.coderight.jazz.action.Action;
import nl.coderight.jazz.action.ActionGroup;
import nl.coderight.jazz.demo.action.AboutAction;
import nl.coderight.jazz.demo.action.AddContactAction;
import nl.coderight.jazz.demo.action.DeleteContactAction;
import nl.coderight.jazz.demo.action.EditContactAction;
import nl.coderight.jazz.demo.action.HelpAction;
import nl.coderight.jazz.demo.contact.ContactFilter;
import nl.coderight.jazz.demo.domain.Contact;
import nl.coderight.jazz.demo.event.ContactEvent;
import nl.coderight.jazz.demo.event.ContactEvent.ContactEventType;
import nl.coderight.jazz.demo.ext.XStatusBar;

import nl.coderight.jazz.dialog.MessageType;
import nl.coderight.jazz.dialog.OptionDialog;
import nl.coderight.jazz.dialog.OptionType;
import nl.coderight.jazz.dialog.ReturnOptionType;
import nl.coderight.jazz.event.ApplicationEvent;
import nl.coderight.jazz.event.ApplicationEvent.ApplicationEventType;
import nl.coderight.jazz.form.FormView;
import nl.coderight.jazz.form.control.GroupControl;
import nl.coderight.jazz.form.field.MemoField;
import nl.coderight.jazz.form.field.SearchField;
import nl.coderight.jazz.form.field.TableField;
import nl.coderight.jazz.form.field.TableField.AutoResizeMode;
import nl.coderight.jazz.form.field.button.PushButton;
import nl.coderight.jazz.help.HelpID;
import nl.coderight.jazz.help.HelpSet;
import nl.coderight.jazz.i18n.Message;
import nl.coderight.jazz.i18n.Message.Choice;
import nl.coderight.jazz.i18n.Message.Param;
import nl.coderight.jazz.util.JazzUtil;
import nl.coderight.jazz.view.Menu;
import nl.coderight.jazz.view.MenuBar;
import nl.coderight.jazz.view.MenuItem;
import nl.coderight.jazz.view.ToolBar;

@HelpSet("nl/coderight/jazz/demo/help/Simple.hs") // load HelpSet and enable help(key)
public class SimpleView extends FormView {
	private Action helpAction;
	private Action aboutAction;

	private Action addContactAction;
	private Action editContactAction;
	private Action deleteContactAction;
	private ActionGroup editContactActionGroup;

	@HelpID("intro") // set context-sensitive HelpID for menu item
	private MenuItem helpMenuItem;

	@HelpID("bean.story") // set context-sensitive HelpID for field
	private SearchField filterField;

	private TableField<Contact> contactTable;
	private ContactFilter contactFilter;
	private MemoField notesField;
	
	private XStatusBar statusBar;

	
	public SimpleView() {
		setTitle("SimpleApp");
		setSize(800, 600);
		center();
		// create filter for contacts table
		contactFilter = new ContactFilter();
	}
	
	public void handleEvent(ContactEvent evt) {
		// get selected contact from contacts table
		Contact selectedContact = contactTable.getValue();
		switch(evt.getType()) {
		// update status bar and notesField when:
		case CHANGE:
			// contacts table is changed (filter)
		case SELECT:
			// contact is selected
			ArrayList<Choice> choices = new ArrayList<Choice>(); 
			choices.add(new Choice(0, "status.message.none"));
			choices.add(new Choice(1, "status.message.one"));
			choices.add(new Choice(2, "status.message.all"));
			// forget MessageFormat, the power of Message!
			Message status = new Message("status.message");
			status.addParam(new Param(contactTable.getValueCount()));
			status.addParam(new Param(contactTable.getListOfValues().size(), choices));
			status.addParam(new Param(contactTable.getSelectionCount()));
			// display message on status bar
			statusBar.setMessage(status);
			break;

		case DELETE:
			// show confirmation dialog
			Message message = new Message("dialog.delete.message");
			message.addParam(new Param(selectedContact.getFirstname()));
			message.addParam(new Param(selectedContact.getLastname()));
			
			OptionDialog dialog = new OptionDialog(this);
			dialog.setTitle("dialog.delete.title");
			dialog.setOptionType(OptionType.YES_NO);
			dialog.setOptions("button.yes", "button.no", "button.cancel");
			dialog.setDefaultOption("button.no");
			dialog.setMessageType(MessageType.QUESTION);
			dialog.setMessage(message);
			dialog.show();			
			if (dialog.getReturnOptionType() == ReturnOptionType.YES) {
				propagateEvent(evt);
			}
			break;
		
		case FILTER:
			// filterField is changed, update filter
			contactFilter.setValue(filterField.getValue());
			break;
			
		default:
			// propagate event to controller
			propagateEvent(evt);
			break;
		}
	}

	protected void init() {
		createActions();
		createFields();
		createLayout();
		
		setMenuBar(createMenuBar());
		setToolBar(createToolBar());

		statusBar = new XStatusBar();
		statusBar.setUsername("CodeRight");
		setStatusBar(statusBar);
		// forward progress events to status bar
		setProgressMonitor(statusBar);
	}
	
	private void createActions() {
		helpAction = new HelpAction();
		aboutAction = new AboutAction();

		addContactAction = new AddContactAction();
		editContactAction = new EditContactAction();
		deleteContactAction = new DeleteContactAction();
		// create group for edit and delete actions
		editContactActionGroup = new ActionGroup();
		editContactActionGroup.addAction(editContactAction);
		editContactActionGroup.addAction(deleteContactAction);
	}

	private void createFields() {
		filterField = new SearchField(25);
		filterField.setInputPrompt("filter.prompt");
		filterField.setOnSearchEvent(new ContactEvent(ContactEventType.FILTER));
		
		// setup table
		contactTable = new TableField<Contact>("contacts", "contact");
		contactTable.setTitle("header.table");
		contactTable.setAutoResizeMode(AutoResizeMode.AUTOFIT);
		contactTable.setAutofitColumn("address.street");
		// set events
		contactTable.setOnSelectEvent(new ContactEvent(ContactEventType.SELECT));
		contactTable.setOnChangeEvent(new ContactEvent(ContactEventType.CHANGE));
		contactTable.setOnEditEvent(new ContactEvent(ContactEventType.EDIT));
		contactTable.setOnSelectGroup(editContactActionGroup);		
		// add columns
		contactTable.addColumn("label.firstname", "firstname");
		contactTable.addColumn("label.lastname", "lastname");
		contactTable.addColumn("label.street", "address.street");
		contactTable.addColumn("label.city", "address.city");
		contactTable.addColumn("label.region", "address.region");
		contactTable.addColumn("label.postal", "address.postalcode");
		// add filter
		contactTable.addFilter(contactFilter);
		// default sort table on contact's last name 
		contactTable.sort("lastname", SortOrder.ASCENDING);
		// set relation for notesField
		contactTable.setRelationID("notes");

		notesField = new MemoField("notes");
		notesField.setEditable(false);
		notesField.setTitle("Notes");
		notesField.setRows(5);
	}

	private void createLayout() {
		addField(createFilterGroup(), NORTH);
		addField(createFieldGroup(), CENTER);
	}

	private GroupControl createFilterGroup() {
		GroupControl filterGroup = new GroupControl();
		filterGroup.setLayout(new FlowLayout(FlowLayout.RIGHT));
		filterGroup.addLabel("filter.label");
		filterGroup.addField(filterField);		
		return filterGroup;
	}

	private GroupControl createFieldGroup() {		
		GroupControl fieldGroup = new GroupControl();
		fieldGroup.addField(contactTable, NORTH);
		fieldGroup.addField(notesField, CENTER);	
		return fieldGroup;
	}
	
	private ToolBar createToolBar() {
		PushButton addButton = new PushButton(addContactAction);
		addButton.setToolTipText(addContactAction.getLongDescription());
		addButton.hideActionText();
		
		PushButton editButton = new PushButton(editContactAction);
		editButton.setToolTipText(editContactAction.getLongDescription());
		editButton.hideActionText();
		
		PushButton deleteButton = new PushButton(deleteContactAction);
		deleteButton.setToolTipText(deleteContactAction.getLongDescription());
		deleteButton.hideActionText();
		
		ToolBar toolBar = new ToolBar(this);
		toolBar.addButton(addButton);
		toolBar.addButton(editButton);
		toolBar.addButton(deleteButton);
		return toolBar;
	}

	private MenuBar createMenuBar() {
		MenuItem openMenu = new MenuItem("menu.item.open");
		
		MenuItem exitMenu = new MenuItem("menu.item.exit");
		exitMenu.setEvent(new ApplicationEvent(ApplicationEventType.EXIT));
		exitMenu.setAccelerator(JazzUtil.createAccelerator("ctrl Q"));
		
		Menu fileMenu = new Menu("menu.file");
		fileMenu.addMenuItem(openMenu);
		fileMenu.addSeparator();
		fileMenu.addMenuItem(exitMenu);		

		Menu editMenu = new Menu("menu.edit");
		editMenu.addMenuItem(new MenuItem(addContactAction));
		editMenu.addMenuItem(new MenuItem(editContactAction));
		editMenu.addMenuItem(new MenuItem(deleteContactAction));
		
		helpMenuItem = new MenuItem(helpAction);
			
		Menu helpMenu = new Menu("menu.help");
		helpMenu.addMenuItem(helpMenuItem);
		helpMenu.addSeparator();
		helpMenu.addMenuItem(new MenuItem(aboutAction));

		MenuBar menuBar = new MenuBar(this);
		menuBar.addMenu(fileMenu);
		menuBar.addMenu(editMenu);
		menuBar.addMenu(helpMenu);		
		return menuBar;
	}
	
}
